@@ -1,5 +1,7 @@
+"""detectree init."""
 from .classifier import Classifier, ClassifierTrainer
+from .lidar import LidarToCanopy, rasterize_lidar
 from .train_test_split import TrainingSelector
 from .utils import split_into_tiles
 
-__version__ = '0.3.1'
+__version__ = "0.4.0"
