@@ -1,3 +1,4 @@
+"""Binary tree/non-tree classifier(s)."""
 import glob
 from os import path
 
@@ -10,20 +11,32 @@ from sklearn import ensemble
 
 from . import pixel_features, pixel_response, settings, utils
 
-__all__ = ['ClassifierTrainer', 'Classifier']
+__all__ = ["ClassifierTrainer", "Classifier"]
 
 MOORE_NEIGHBORHOOD_ARR = np.array([[0, 0, 0], [0, 0, 1], [1, 1, 1]])
 
 
 class ClassifierTrainer(object):
-    def __init__(self, num_estimators=None, sigmas=None, num_orientations=None,
-                 neighborhood=None, min_neighborhood_range=None,
-                 num_neighborhoods=None, tree_val=None, nontree_val=None,
-                 **adaboost_kws):
+    """Train binary tree/non-tree classifier(s) of the pixel features."""
+
+    def __init__(
+        self,
+        *,
+        num_estimators=None,
+        sigmas=None,
+        num_orientations=None,
+        neighborhood=None,
+        min_neighborhood_range=None,
+        num_neighborhoods=None,
+        tree_val=None,
+        nontree_val=None,
+        **adaboost_kws,
+    ):
         """
-        Class to train a binary tree/non-tree classifier(s) of the pixel
-        features. See the `background <https://bit.ly/2KlCICO>`_ example
-        notebook for more details.
+        Initialize the classifier.
+
+        See the `background <https://bit.ly/2KlCICO>`_ example notebook for
+        more details.
 
         Parameters
         ----------
@@ -83,7 +96,6 @@ class ClassifierTrainer(object):
             Keyword arguments that will be passed to
             `sklearn.ensemble.AdaBoostClassifier`.
         """
-
         super(ClassifierTrainer, self).__init__()
 
         if num_estimators is None:
@@ -91,21 +103,34 @@ class ClassifierTrainer(object):
         self.num_estimators = num_estimators
 
         self.pixel_features_builder_kws = dict(
-            sigmas=sigmas, num_orientations=num_orientations,
+            sigmas=sigmas,
+            num_orientations=num_orientations,
             neighborhood=neighborhood,
             min_neighborhood_range=min_neighborhood_range,
-            num_neighborhoods=num_neighborhoods)
-        self.pixel_response_builder_kws = dict(tree_val=tree_val,
-                                               nontree_val=nontree_val)
+            num_neighborhoods=num_neighborhoods,
+        )
+        self.pixel_response_builder_kws = dict(
+            tree_val=tree_val, nontree_val=nontree_val
+        )
         self.adaboost_kws = adaboost_kws
 
-    def train_classifier(self, split_df=None, response_img_dir=None,
-                         img_filepaths=None, response_img_filepaths=None,
-                         img_dir=None, img_filename_pattern=None, method=None,
-                         img_cluster=None):
+    def train_classifier(
+        self,
+        *,
+        split_df=None,
+        response_img_dir=None,
+        img_filepaths=None,
+        response_img_filepaths=None,
+        img_dir=None,
+        img_filename_pattern=None,
+        method=None,
+        img_cluster=None,
+    ):
         """
-        Train a classifier. See the `background <https://bit.ly/2KlCICO>`_
-        example notebook for more details.
+        Train a classifier.
+
+        See the `background <https://bit.ly/2KlCICO>`_ example notebook for
+        more details.
 
         Parameters
         ----------
@@ -154,14 +179,17 @@ class ClassifierTrainer(object):
                 # TODO: this is copied from `build_features` - ideally, we
                 # should DRY it
                 if img_filename_pattern is None:
-                    img_filename_pattern = \
+                    img_filename_pattern = (
                         settings.IMG_DEFAULT_FILENAME_PATTERN
+                    )
                 if img_dir is None:
                     raise ValueError(
                         "Either `split_df`, `img_filepaths` or `img_dir` must "
-                        "be provided")
+                        "be provided"
+                    )
                 img_filepaths = glob.glob(
-                    path.join(img_dir, img_filename_pattern))
+                    path.join(img_dir, img_filename_pattern)
+                )
 
             response_img_filepaths = [
                 path.join(response_img_dir, path.basename(img_filepath))
@@ -169,29 +197,40 @@ class ClassifierTrainer(object):
             ]
 
         X = pixel_features.PixelFeaturesBuilder(
-            **self.pixel_features_builder_kws).build_features(
-                split_df=split_df, img_filepaths=img_filepaths,
-                img_dir=img_dir, img_filename_pattern=img_filename_pattern,
-                method=method, img_cluster=img_cluster)
+            **self.pixel_features_builder_kws
+        ).build_features(
+            split_df=split_df,
+            img_filepaths=img_filepaths,
+            img_dir=img_dir,
+            img_filename_pattern=img_filename_pattern,
+            method=method,
+            img_cluster=img_cluster,
+        )
 
         y = pixel_response.PixelResponseBuilder(
-            **self.pixel_response_builder_kws).build_response(
-                split_df=split_df, response_img_dir=response_img_dir,
-                response_img_filepaths=response_img_filepaths,
-                img_filename_pattern=img_filename_pattern, method=method,
-                img_cluster=img_cluster)
+            **self.pixel_response_builder_kws
+        ).build_response(
+            split_df=split_df,
+            response_img_dir=response_img_dir,
+            response_img_filepaths=response_img_filepaths,
+            img_filename_pattern=img_filename_pattern,
+            method=method,
+            img_cluster=img_cluster,
+        )
 
-        clf = ensemble.AdaBoostClassifier(n_estimators=self.num_estimators,
-                                          **self.adaboost_kws)
+        clf = ensemble.AdaBoostClassifier(
+            n_estimators=self.num_estimators, **self.adaboost_kws
+        )
         clf.fit(X, y)
 
         return clf
 
     def train_classifiers(self, split_df, response_img_dir):
         """
-        Train a classifier for each first-level cluster in `split_df`. See the
-        `background <https://bit.ly/2KlCICO>`_ example notebook for more
-        details.
+        Train a classifier for each first-level cluster in `split_df`.
+
+        See the `background <https://bit.ly/2KlCICO>`_ example notebook for
+        more details.
 
         Parameters
         ----------
@@ -207,16 +246,20 @@ class ClassifierTrainer(object):
             Dictionary mapping a scikit-learn AdaBoostClassifier to each
             first-level cluster label
         """
-        if 'img_cluster' not in split_df:
+        if "img_cluster" not in split_df:
             raise ValueError(
                 "`split_df` must have an 'img_cluster' column ('cluster-II'). "
-                "For 'cluster-I', use `train_classifier`.")
+                "For 'cluster-I', use `train_classifier`."
+            )
 
         clfs_lazy = {}
-        for img_cluster, _ in split_df.groupby('img_cluster'):
+        for img_cluster, _ in split_df.groupby("img_cluster"):
             clfs_lazy[img_cluster] = dask.delayed(self.train_classifier)(
-                split_df=split_df, response_img_dir=response_img_dir,
-                method='cluster-II', img_cluster=img_cluster)
+                split_df=split_df,
+                response_img_dir=response_img_dir,
+                method="cluster-II",
+                img_cluster=img_cluster,
+            )
 
         with diagnostics.ProgressBar():
             clfs_dict = dask.compute(clfs_lazy)[0]
@@ -225,11 +268,21 @@ class ClassifierTrainer(object):
 
 
 class Classifier(object):
-    def __init__(self, tree_val=None, nontree_val=None, refine=None,
-                 refine_beta=None, refine_int_rescale=None,
-                 **pixel_features_builder_kws):
+    """Use trained classifier(s) to predict tree pixels."""
+
+    def __init__(
+        self,
+        *,
+        tree_val=None,
+        nontree_val=None,
+        refine=None,
+        refine_beta=None,
+        refine_int_rescale=None,
+        **pixel_features_builder_kws,
+    ):
         """
-        Class use the trained classifier(s) to classify tree/non-tree pixels.
+        Initialize the classifier instance.
+
         See the `background <https://bit.ly/2KlCICO>`_ example notebook for
         more details.
 
@@ -287,8 +340,9 @@ class Classifier(object):
 
     def classify_img(self, img_filepath, clf, output_filepath=None):
         """
-        Classify the image in `img_filepath` with the classifier `clf`, and
-        optionally dump it to `output_filepath`.
+        Use a trained classifier to predict tree pixels in an image.
+
+        Optionally dump the predicted tree/non-tree image to `output_filepath`.
 
         Parameters
         ----------
@@ -305,16 +359,18 @@ class Classifier(object):
 
         Returns
         -------
-        y_pred : np.ndarray
+        y_pred : numpy ndarray
             Array with the pixel responses
         """
-
+        # ACHTUNG: Note that we do not use keyword-only arguments in this
+        # method because `output_filepath` works as the only "optional"
+        # argument
         src = rio.open(img_filepath)
         img_shape = src.shape
 
         X = pixel_features.PixelFeaturesBuilder(
-            **self.pixel_features_builder_kws).build_features_from_filepath(
-                img_filepath)
+            **self.pixel_features_builder_kws
+        ).build_features_from_filepath(img_filepath)
 
         if not self.refine:
             y_pred = clf.predict(X).reshape(img_shape)
@@ -339,8 +395,9 @@ class Classifier(object):
             D_tree = (self.refine_int_rescale * np.log(P_nontree)).astype(int)
             D_nontree = (self.refine_int_rescale * np.log(P_tree)).astype(int)
             # TODO: option to choose Moore/Von Neumann neighborhood?
-            g.add_grid_edges(node_ids, self.refine_beta,
-                             structure=MOORE_NEIGHBORHOOD_ARR)
+            g.add_grid_edges(
+                node_ids, self.refine_beta, structure=MOORE_NEIGHBORHOOD_ARR
+            )
             g.add_grid_tedges(node_ids, D_tree, D_nontree)
             g.maxflow()
             # y_pred = g.get_grid_segments(node_ids)
@@ -354,10 +411,18 @@ class Classifier(object):
         # output_filepath = path.join(output_dir,
         #                             f"tile_{tile_start}-{tile_end}.tif")
         if output_filepath is not None:
-            with rio.open(output_filepath, 'w', driver='GTiff',
-                          width=y_pred.shape[1], height=y_pred.shape[0],
-                          count=1, dtype=np.uint8, nodata=self.nontree_val,
-                          crs=src.crs, transform=src.transform) as dst:
+            with rio.open(
+                output_filepath,
+                "w",
+                driver="GTiff",
+                width=y_pred.shape[1],
+                height=y_pred.shape[0],
+                count=1,
+                dtype=np.uint8,
+                nodata=self.nontree_val,
+                crs=src.crs,
+                transform=src.transform,
+            ) as dst:
                 dst.write(y_pred.astype(np.uint8), 1)
 
         src.close()
@@ -370,11 +435,14 @@ class Classifier(object):
             # filename, ext = path.splitext(path.basename(img_filepath))
             # pred_img_filepath = path.join(
             #     output_dir, f"{filename}-pred{ext}")
-            pred_img_filepath = path.join(output_dir,
-                                          path.basename(img_filepath))
+            pred_img_filepath = path.join(
+                output_dir, path.basename(img_filepath)
+            )
             pred_imgs_lazy.append(
-                dask.delayed(self.classify_img)(img_filepath, clf,
-                                                pred_img_filepath))
+                dask.delayed(self.classify_img)(
+                    img_filepath, clf, pred_img_filepath
+                )
+            )
             pred_img_filepaths.append(pred_img_filepath)
 
         with diagnostics.ProgressBar():
@@ -383,23 +451,26 @@ class Classifier(object):
         return pred_img_filepaths
 
     def classify_imgs(
-            self,
-            split_df,
-            output_dir,
-            clf=None,
-            clf_dict=None,
-            method=None,
-            img_cluster=None,
+        self,
+        split_df,
+        output_dir,
+        *,
+        clf=None,
+        clf_dict=None,
+        method=None,
+        img_cluster=None,
     ):
         """
-        Classify thes image in `img_filepaths` with the classifier(s) `clf` or
-        `clf_dict` (depending on the train/test split method) dump them to
+        Use trained classifier(s) to predict tree pixels in multiple images.
+
+        Use `clf` or `clf_dict` for the classifier(s) depending on the train/
+        test split method, and dump the predicted tree/non-tree images to
         `output_dir`. See the `background <https://bit.ly/2KlCICO>`_ example
         notebook for more details.
 
 
         Parameters
-        -------
+        ----------
         split_df : pandas DataFrame, optional
             Data frame with the train/test split.
         output_dir : str or pathlib.Path object
@@ -420,19 +491,20 @@ class Classifier(object):
         pred_imgs : list or dict
             File paths of the dumped tiles.
         """
-
         if method is None:
-            if 'img_cluster' in split_df:
-                method = 'cluster-II'
+            if "img_cluster" in split_df:
+                method = "cluster-II"
             else:
-                method = 'cluster-I'
+                method = "cluster-I"
 
-        if method == 'cluster-I':
+        if method == "cluster-I":
             if clf is None:
                 raise ValueError(
-                    "If using 'cluster-I' method, `clf` must be provided")
+                    "If using 'cluster-I' method, `clf` must be provided"
+                )
             return self._classify_imgs(
-                split_df[~split_df['train']]['img_filepath'], clf, output_dir)
+                split_df[~split_df["train"]]["img_filepath"], clf, output_dir
+            )
         else:
             if img_cluster is not None:
                 if clf is None:
@@ -440,20 +512,26 @@ class Classifier(object):
                         clf = clf_dict[img_cluster]
                     else:
                         raise ValueError(
-                            "Either `clf` or `clf_dict` must be provided")
+                            "Either `clf` or `clf_dict` must be provided"
+                        )
 
                 return self._classify_imgs(
-                    utils.get_img_filepaths(split_df, img_cluster, False), clf,
-                    output_dir)
+                    utils.get_img_filepaths(split_df, img_cluster, False),
+                    clf,
+                    output_dir,
+                )
 
             if clf_dict is None:
                 raise ValueError(
                     "If using 'cluster-II' method and not providing "
-                    "`img_cluster`, `clf_dict` must be provided")
+                    "`img_cluster`, `clf_dict` must be provided"
+                )
             pred_imgs = {}
-            for img_cluster, img_cluster_df in split_df.groupby('img_cluster'):
+            for img_cluster, img_cluster_df in split_df.groupby("img_cluster"):
                 pred_imgs[img_cluster] = self._classify_imgs(
                     utils.get_img_filepaths(split_df, img_cluster, False),
-                    clf_dict[img_cluster], output_dir)
+                    clf_dict[img_cluster],
+                    output_dir,
+                )
 
             return pred_imgs
